<?php
/*
* 2007-2024 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/
require_once(dirname(__FILE__).'../../../classes/CardValidation.php');
require_once(dirname(__FILE__).'../../../classes/TcbpayGateway.php');
/**
 * @since 1.6.0
 */
class Tcbpay_PaymentPaymentModuleFrontController extends ModuleFrontController
{
	public $ssl = true;

	/**
	 * @see FrontController::postProcess()
	 */
	public function postProcess()
	{
		$cart = $this->context->cart;
		if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active)
			Tools::redirect('index.php?controller=order&step=1');

		// Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
		$authorized = false;
		foreach (Module::getPaymentModules() as $module)
			if ($module['name'] == 'tcbpay_payment')
			{
				$authorized = true;
				break;
			}
		if (!$authorized) {
			$this->errors[]=$this->module->l('This payment method is not available.');
		}
		$customer = new Customer($cart->id_customer);
		if (!Validate::isLoadedObject($customer))
			$this->errors[]=$this->module->l('Customer not found.');

		$tcbpayGateway = new TcbpayGateway();
		$ccNumber = preg_replace('/[\-\s]+/', '', Tools::getValue('cardnumber'));
		$encCardNum = trim(Tools::getValue('enccardnum'));
		$ccExpMonth = trim(Tools::getValue('expires_month'));
		$ccExpYear = trim(Tools::getValue('expires_year'));
		$cardCVV = '';
		$currency = $this->context->currency;
		$total = (float)$cart->getOrderTotal(true, Cart::BOTH);
		$error = false;
		if(Configuration::get('TCBPAY_USECVV')) {
			$cardCVV = Tools::getValue('cardcvv');
		}
		if(!Tools::getValue('cardholder')) {
			$this->errors[] = $this->module->l('The credit card holder\'s name is required.');
		}
		if(!Tools::getValue('enccardnum')) {
			$this->errors[] = $this->module->l('The credit card number is required.');
		} else {
			$response = $tcbpayGateway->decodeCard(Tools::getValue('enccardnum'));
			$dcresp = $response['resp'];

			if ($response['success']) {
				$ccNumber = preg_replace('/[\-\s]+/', '', trim($dcresp->accountNumber));
			} elseif($dcresp) {
				$this->errors[] = $dcresp->description;
			} else {
				$this->errors[] = 'TCBPay gateway error.';
			}
		}
		if(Configuration::get('TCBPAY_USECVV') && !$cardCVV) {
			$this->errors[] = $this->module->l('The cvv is required.');
		}

		if(sizeof($this->errors)) {
			$this->redirectWithNotifications('index.php?controller=order&step=4');
		}

		$cardvalidation = new TcbpayCardValidation();
		$cvld = $cardvalidation->validate($ccNumber, $ccExpMonth, $ccExpYear);

        switch (true) {
			case $cvld === -1:
				$this->errors[] = sprintf($this->module->l('The first four digits of the number entered are: %s. If that number is correct, we do not accept that type of credit card. If it is wrong, please try again.'), substr($cardvalidation->cc_number, 0, 4));
				break;
			case $cvld === -2:
			case $cvld === -3:
			case $cvld === -4:
				$this->errors[] = $this->module->l('The expiry date entered for the credit card is invalid. Please check the date and try again.');
				break;
			case $cvld === -5:
			case $cvld === -6:
			case $cvld === -7:
				$this->errors[] = $this->module->l('Card Verification Value is invalid.');
				break;
			case $cvld === -8:
				$this->errors[] = $this->module->l('We do not accept the type of credit card you have entered.');
				break;
			case $cvld === false:
				$this->errors[] = $this->module->l('The credit card number entered is invalid. Please check the number and try again.');
				break;
        }
		if(!in_array($cardvalidation->cc_type, explode(',',Configuration::get('TCBPAY_CCTYPES')))) {
			$this->errors[] = $this->module->l('We do not accept the type of credit card you have entered.');
		}
		if(Configuration::get('TCBPAY_USECVV') && !preg_match($cardvalidation->getCvvRegEx()[$cardvalidation->cc_type], $cardCVV)) {
			$this->errors[] = $this->module->l('The credit card verification number is invalid.');
		}

		if (empty($this->errors)) {
			$billing = new Address($cart->id_address_invoice);
	        $state_code = '';
	        $state = new State($billing->id_state);
	        if (Validate::isLoadedObject($state)) {
	            $state_code = $state->iso_code;
	        }
	        $memo = '';
	        if ($message = Message::getMessageByCartId($cart->id)) {
	            $memo = $message['message'];
	        }
	        $post_data = array(
	            "amount"                => $total,
	            "creditcard_number"     => $encCardNum,                
	            "expire_month"          => str_pad($ccExpMonth, 2, 0, STR_PAD_LEFT),
	            "expire_year"           => $ccExpYear,
	            "cvv"                   => $cardCVV,
	            "first_name"            => $billing->firstname,
	            "last_name"             => $billing->lastname,
	            "address"               => $billing->address1,
	            "city"                  => $billing->city,
	            "state"                 => $state_code,
	            "zip_code"              => $billing->postcode,
	            "country_code"          => Country::getIsoById($billing->id_country),
	            "phone_number"          => $billing->phone,
	            "email"                 => $customer->email,
	            "currency"              => $currency->iso_code,
	            "memo"        			=> $memo
	        );
			$response = $tcbpayGateway->sendRequest($post_data);
			$resp = $response['resp'];

			if ($response['success']) {
				$this->module->validateOrder($cart->id, Configuration::get('PS_OS_PAYMENT'), $total, $this->module->l('Credit Card(TCB Pay)'), 'TCB Pay Transaction Id: ' . $resp->transaction_id, ['transaction_id'=>$resp->transaction_id], (int)$currency->id, false, $customer->secure_key);
				Tools::redirect('index.php?controller=order-confirmation&id_cart='.$cart->id.'&id_module='.$this->module->id.'&id_order='.$this->module->currentOrder.'&key='.$customer->secure_key);
			} elseif($resp) {
				$this->errors[] = $resp->description;
			} else {
				$this->errors[] = $this->module->l('Error occurred during payment process. Try after sometime.');
			}
		}

		//$this->errors[] = $this->module->l('Error occurred during payment process. Try after sometime.');
		$this->redirectWithNotifications('index.php?controller=order&step=4');
	}
}
