<?php
/*
* 2007-2024 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Tcbpay_Payment extends PaymentModule
{
    const API_LIVE_URL     = 'https://gateway2.tcbpay.com/mobile_plugin/sale.php';
    const API_TEST_URL     = 'https://gateway2-stg.tcbpay.com/mobile_plugin/sale.php';

    protected $_html = '';
    protected $_postErrors = array();
    protected $errors = array();

    public function __construct()
    {
        $this->name = 'tcbpay_payment';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => '1.7.9.99');
        $this->author = 'TCB Pay';
        $this->controllers = array('payment', 'validation');
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        $this->bootstrap = true;

        /* curl check */
        if (is_callable('curl_init') === false) {
            $this->errors[] = $this->l('To be able to use this module, please activate cURL (PHP extension).');
        }
        parent::__construct();

        $this->displayName = $this->l('TCB Pay');
        $this->description = $this->l('Process payments through TCB Pay.');
        $this->confirmUninstall = $this->l('Are you sure about removing these details?');
        if (!isset($this->owner) || !isset($this->details) || !isset($this->address)) {
            $this->warning = $this->l('Account owner and account details must be configured before using this module.');
        }
        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install()
    {
        Configuration::updateValue('TCBPAY_ACTIVE', 1);
        Configuration::updateValue('TCBPAY_TESTMODE', 1);
        Configuration::updateValue('TCBPAY_USECVV', 1);
        if (!parent::install() || !$this->registerHook('header') || !$this->registerHook('paymentOptions') || !$this->registerHook('payment')) {
            return false;
        }
        return true;
    }

    public function uninstall()
    {
        if (!Configuration::deleteByName('TCBPAY_ACTIVE')
                || !Configuration::deleteByName('TCBPAY_TESTMODE')
                || !Configuration::deleteByName('TCBPAY_USERNAME')
                || !Configuration::deleteByName('TCBPAY_PASSWORD')
                || !Configuration::deleteByName('TCBPAY_SECRETKEY')
                || !Configuration::deleteByName('TCBPAY_CCTYPES')
                || !Configuration::deleteByName('TCBPAY_USECVV')
                || !parent::uninstall()) {
            return false;
        }
        return true;
    }

    protected function _postValidation()
    {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('TCBPAY_USERNAME')) {
                $this->_postErrors[] = $this->l('Merchant Username is required.');
            } elseif (!Tools::getValue('TCBPAY_PASSWORD')) {
                $this->_postErrors[] = $this->l('API Key is required.');
            } elseif (!Tools::getValue('TCBPAY_SECRETKEY')) {
                $this->_postErrors[] = $this->l('Secret Key is required.');
            } elseif (!Tools::getValue('TCBPAY_CCTYPES')) {
                $this->_postErrors[] = $this->l('Credit card types is required.');
            }
        }
    }

    protected function _postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('TCBPAY_ACTIVE', Tools::getValue('TCBPAY_ACTIVE'));
            Configuration::updateValue('TCBPAY_TESTMODE', Tools::getValue('TCBPAY_TESTMODE'));
            Configuration::updateValue('TCBPAY_USERNAME', Tools::getValue('TCBPAY_USERNAME'));
            Configuration::updateValue('TCBPAY_PASSWORD', Tools::getValue('TCBPAY_PASSWORD'));
            Configuration::updateValue('TCBPAY_SECRETKEY', Tools::getValue('TCBPAY_SECRETKEY'));
            Configuration::updateValue('TCBPAY_USECVV', Tools::getValue('TCBPAY_USECVV'));
            Configuration::updateValue('TCBPAY_CCTYPES', implode(',',Tools::getValue('TCBPAY_CCTYPES')));
        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    protected function _displayTcbPay()
    {
        return $this->fetch('module:'.$this->name.'/views/templates/admin/settings_info.tpl');
    }

    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }
        } elseif (count($this->errors)) {
            foreach ($this->errors as $err) {
                $this->_html .= $this->displayError($err);
            }
        }

        $this->_html .= $this->_displayTcbPay();
        $this->_html .= $this->renderForm();

        return $this->_html;
    }

    public function hookHeader()
    {
        if (!in_array($this->context->controller->php_self, ['order', 'order-opc'])) {
            return;
        }

        if (!$this->active || ($this->active && !Configuration::get('TCBPAY_ACTIVE'))) {
            return;
        }

        $amount = $this->context->cart->getOrderTotal();
        $amount = Tools::ps_round($amount, 2);

        if ($amount <= 0) {
            return;
        }

        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $this->context->controller->registerStylesheet(
                $this->name.'-css',
                'modules/' . $this->name . '/views/css/styles.css'
            );
            $jsurl = 'https://gateway2.tcbpay.com/services/proxynization_api.js';
            if(Configuration::get('TCBPAY_TESTMODE')) {
                $jsurl = 'https://gateway2-stg.tcbpay.com/services/proxynization_api.js';
            }
            $this->context->controller->registerJavascript(
                $this->name . '-js',
                $jsurl,
                ['position' => 'bottom', 'priority' => 170, 'server' => 'remote']
            );
            $this->context->controller->registerJavascript(
                $this->name . '-formjs',
                'modules/' . $this->name . '/views/js/main.js',
                ['position' => 'bottom', 'priority' => 180]
            );
        } else {
            $this->context->controller->addCSS($this->_path . '/views/css/styles.css', 'all');
        }
    }

    public function hookPaymentOptions($params)
    {
        if (!$this->active || ($this->active && !Configuration::get('TCBPAY_ACTIVE'))) {
            return;
        }
        if (!Configuration::get('TCBPAY_USERNAME') || !Configuration::get('TCBPAY_PASSWORD') || !Configuration::get('TCBPAY_SECRETKEY')) {
            return;
        }

        $cart = $params['cart'];

        if (!$this->checkCurrency($cart)) {
            return;
        }

        $newOption = new PaymentOption();
        $newOption->setModuleName($this->name)
                ->setCallToActionText($this->l('Credit Card (TCB Pay)'))
                ->setAction($this->context->link->getModuleLink($this->name, 'payment', array(), true))
                ->setForm($this->creditCardForm());

        $payment_options = [
            $newOption,
        ];

        return $payment_options;
    }

    private function creditCardForm() {
        $cart = $this->context->cart;
        $billing = new Address($cart->id_address_invoice);

        $expires_month = [];
        for ($i=1; $i<13; $i++) {
            $expires_month[] = array('id' => sprintf('%02d', $i), 'name' => strftime('%B',mktime(0,0,0,$i,1,2000)));
        }

        $expires_year = [];
        $today = getdate(); 
        for ($i=$today['year']; $i < $today['year']+10; $i++) {
            $expires_year[] = array('id' => strftime('%y',mktime(0,0,0,1,1,$i)), 'name' => strftime('%Y',mktime(0,0,0,1,1,$i)));
        }

        $this->context->smarty->assign([
            'customer_name' => $billing->firstname . ' ' . $billing->lastname,
            'action' => $this->context->link->getModuleLink($this->name, 'payment', array(), true),
            'expires_month' => $expires_month,
            'expires_year' => $expires_year,
            'usecvv' => Configuration::get('TCBPAY_USECVV'),
        ]);

        return $this->fetch('module:'.$this->name.'/views/templates/front/payment_form.tpl');
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function renderForm()
    {
        $settings_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enabled'),
                        'name' => 'TCBPAY_ACTIVE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'yes',
                                'value' => true,
                                'label' => $this->l('Yes'),
                            ),
                            array(
                                'id' => 'no',
                                'value' => false,
                                'label' => $this->l('No'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Test Mode'),
                        'name' => 'TCBPAY_TESTMODE',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'yes',
                                'value' => true,
                                'label' => $this->l('Yes'),
                            ),
                            array(
                                'id' => 'no',
                                'value' => false,
                                'label' => $this->l('No'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Merchant Username'),
                        'name' => 'TCBPAY_USERNAME',
                        'required' => true,
                        'desc' => $this->l('This is the username provided by TCB Pay when you signed up for an account.'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('API Key'),
                        'name' => 'TCBPAY_PASSWORD',
                        'required' => true,
                        'desc' => $this->l('This is the API Key provided by TCB Pay when you signed up for an account.'),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Secret Key'),
                        'name' => 'TCBPAY_SECRETKEY',
                        'required' => true,
                        'desc' => $this->l('This is the Secret Key provided by TCB Pay when you signed up for an account.'),
                    ),
                    array(
                        'type' => 'select',
                        'multiple' => true,
                        'label' => $this->l('Credit Card Types'),
                        'name' => 'TCBPAY_CCTYPES[]',
                        'id' => 'TCBPAY_CCTYPES',
                        'options' => [
                            'query' => [
                                ['id' => 'AE', 'name' => $this->l('American Express')],
                                ['id' => 'VI', 'name' => $this->l('Visa')],
                                ['id' => 'MC', 'name' => $this->l('MasterCard')],
                                ['id' => 'DI', 'name' => $this->l('Discover')],
                                ['id' => 'JCB', 'name' => $this->l('JCB')],
                                ['id' => 'OT', 'name' => $this->l('Other')],
                            ],
                            'id' => 'id',
                            'name'=> 'name'
                        ]
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Use CVV'),
                        'name' => 'TCBPAY_USECVV',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'yes',
                                'value' => true,
                                'label' => $this->l('Yes'),
                            ),
                            array(
                                'id' => 'no',
                                'value' => false,
                                'label' => $this->l('No'),
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='
            .$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($settings_form));
    }

    public function getConfigFieldsValues()
    {
        $languages = Language::getLanguages(false);

        return array(
            'TCBPAY_ACTIVE' => Tools::getValue('TCBPAY_ACTIVE', Configuration::get('TCBPAY_ACTIVE')),
            'TCBPAY_TESTMODE' => Tools::getValue('TCBPAY_TESTMODE', Configuration::get('TCBPAY_TESTMODE')),
            'TCBPAY_USERNAME' => Tools::getValue('TCBPAY_USERNAME', Configuration::get('TCBPAY_USERNAME')),
            'TCBPAY_PASSWORD' => Tools::getValue('TCBPAY_PASSWORD', Configuration::get('TCBPAY_PASSWORD')),
            'TCBPAY_SECRETKEY' => Tools::getValue('TCBPAY_SECRETKEY', Configuration::get('TCBPAY_SECRETKEY')),
            'TCBPAY_CCTYPES[]' => Tools::getValue('TCBPAY_CCTYPES', explode(',',Configuration::get('TCBPAY_CCTYPES'))),
            'TCBPAY_USECVV' => Tools::getValue('TCBPAY_USECVV', Configuration::get('TCBPAY_USECVV')),
        );
    }

}
